@@ -4,7 +4,7 @@
  * A little RPN (Reverse Polish Notation) calculator, rudimentary          *
  * emulating a HP 28S. 					                   *
  * 								           *
- * rpncalc is (c) David Frey, 1993, 1994, 1995, 1996, 1997		   *
+ * rpncalc is (c) David Frey, 1993, 1994, 1995, 1996, 1997, 1998	   *
  *								           * 
  * This program is free software; you can redistribute it and/or modify it *
  * under the terms of the GNU General Public License as published by the   *
@@ -21,25 +21,20 @@
  * Inc., 59 Temple Place Suite 330, Boston, MA 02111-1307 USA.             *
  ***************************************************************************/
 
-/* $Id: rpncalc.c,v 1.4 1997/01/19 19:09:00 david Rel $
+/* $Id: rpncalc.c,v 1.7 1998/01/04 16:40:53 david Exp david $
  * $Log: rpncalc.c,v $
- * Revision 1.4  1997/01/19 19:09:00  david
- * New command `char' displays character code of a character; help now
- * displays the commands out of the cmdtab[] array.
+ * Revision 1.7  1998/01/04 16:40:53  david
+ * - changed HAVE_READLINE to HAVE_LIBREADLINE, HAVE_HISTORY to
+ *   HAVE_LIBHISTORY for future use with autoconf.
+ * - added the meaningfull X/OPEN-math functions to rpncalc:
+ *   ln1p (log1p), ldb (logb), expm1, gamma, lgamma, hypot, cbrt, floor, rint.
+ *   Changed the names of the log-named functions to reflect their base
+ *   (log -> ln, lobg -> ld)
+ * - --warranty and --license switches are new.
+ * - renamed -v to -V switch.
+ * - changed version to 1.2
  *
- * Revision 1.4  1997/01/19 18:19:23  david
- * Version 1.2: new command `char' displays character code etc of a character,
- * help now displays the commands out of the cmdtab[] array.
- *
- * Revision 1.2  1996/09/13 20:21:29  david
- * lclint additions
- *
- * Revision 1.1  1996/07/13 20:53:44  david
- * Added operator completion, renaming due to linting
- *
- * Revision 1.0  1995/12/31 18:14:13  david
- * Initial revision
- * */ 
+ */ 
 
 #include <stdio.h>
 #include <stdlib.h> 
@@ -47,6 +42,7 @@
 #include <math.h>
 #include <string.h>
 #include <ctype.h>
+#include <errno.h>
 
 #include <signal.h> 
 #include <locale.h>
@@ -55,8 +51,14 @@
 #include <getopt.h>
 #endif
 
-#ifdef HAVE_READLINE
+#ifdef HAVE_LIBREADLINE
 #include <readline/readline.h>
+
+char *command_generator (char *text, int state);
+char **rpncalc_completion(char *text, int start, int end);
+void initialize_readline(int interactive);
+#endif
+#ifdef HAVE_LIBHISTORY
 #include <readline/history.h>
 #endif
 
@@ -66,13 +68,13 @@
 #include "rpncalc.h"
 
 extern int pushtostack;
-extern int errno;
-extern int optind;
 
 char *progname;
 
-#ifndef linux
+#ifndef HAVE_POW10
 extern double pow10(double y);
+#endif
+#ifndef HAVE_POW2
 extern double pow2(double y);
 #endif
 
@@ -125,14 +127,17 @@ struct cmd cmdtab[] =
   { "atan2"   , 2, 1, atan2     },
   { "sinh"    , 1, 1, sinh      },
   { "cosh"    , 1, 1, cosh      },
-  { "tanh"    , 1, 1, tanh      }, 
+  { "tanh"    , 1, 1, tanh      },
   { "asinh"   , 1, 1, asinh     },
   { "acosh"   , 1, 1, acosh     },
   { "atanh"   , 1, 1, atanh     },
   { "ln"      , 1, 1, log       },
   { "log"     , 1, 1, log10     },
+  { "ln1p"    , 1, 1, log1p     },
   { "ld"      , 1, 1, log2      },
+  { "ldb"     , 1, 1, logb      },
   { "exp"     , 1, 1, exp       },
+  { "expm1"   , 1, 1, expm1     },
   { "alog"    , 1, 1, pow10     },
   { "shl"     , 1, 1, pow2      },
   { "j0"      , 1, 1, j0        },
@@ -143,9 +148,14 @@ struct cmd cmdtab[] =
   { "yn"      , 2, 1, yn        },
   { "erf"     , 1, 1, erf       },
   { "erfc"    , 1, 1, erfc      },
+  { "gamma"   , 1, 1, gamma     },
   { "lgamma"  , 1, 1, lgamma    },
+  { "hypot"   , 2, 1, hypot     },
+  { "cbrt"    , 1, 1, cbrt      },
   { "abs"     , 1, 1, fabs      },
   { "ceil"    , 1, 1, ceil      },
+  { "floor"   , 1, 1, floor     },
+  { "rint"    , 1, 1, rint      },
   { "fact"    , 1, 1, fact      },
   { "!"       , 1, 1, fact      },
   { "mod"     , 2, 1, mod       },
@@ -176,9 +186,11 @@ struct cmd cmdtab[] =
 #ifdef HAVE_GETOPT_LONG
 struct option const long_options[] =
 {
-  {"help",    no_argument, 0, 'h'},
-  {"help",    no_argument, 0, '?'},
-  {"version", no_argument, 0, 'v'},
+  {"help",     no_argument, 0, 'h'},
+  {"help",     no_argument, 0, '?'},
+  {"version",  no_argument, 0, 'V'},
+  {"warranty", no_argument, 0, 'W'},
+  {"license",  no_argument, 0, 'W'},
   {(char *)0, 0,           0, (char)0}
 };
 #endif
@@ -298,23 +310,13 @@ static void usage(void)
   fprintf(stderr, "usage: %s [-h][-v] [\"expression\"]\n", progname);
 }
 
-#ifdef HAVE_READLINE
+#ifdef HAVE_LIBREADLINE
 /* the initialisation code was taken from:
      fileman.c -- A tiny application which demonstrates how to use the
      GNU Readline library.  This application interactively allows users
      to manipulate files and their modes. 
  */
 
-/* Attempt to complete on the contents of TEXT.  START and END show the
-   region of TEXT that contains the word to complete.  We can use the
-   entire line in case we want to do some simple parsing.  Return the
-   array of matches, or NULL if there aren't any. */
-
-char **rpncalc_completion(char *text, int start, int end)
-{
-  return completion_matches (text, command_generator);
-}
-
 /* Generator function for command completion.  STATE lets us know whether
    to start from scratch; without any state (i.e. STATE == 0), then we
    start at the top of the list. */
@@ -339,6 +341,16 @@ char *command_generator (char *text, int state)
   return ((char *)NULL);
 }
 
+/* Attempt to complete on the contents of TEXT.  START and END show the
+   region of TEXT that contains the word to complete.  We can use the
+   entire line in case we want to do some simple parsing.  Return the
+   array of matches, or NULL if there aren't any. */
+
+char **rpncalc_completion(char *text, int start, int end)
+{
+  return completion_matches (text, command_generator);
+}
+
 /* Tell the GNU Readline library how to complete.  We want to try to complete
    on command names if this is the first word in the line. */
 void initialize_readline(int interactive)
@@ -369,23 +381,25 @@ int main(int argc, char *argv[])
   else progname++;
 
 #ifdef HAVE_GETOPT_LONG
-  while ((c = getopt_long(argc, argv, "h?v",
+  while ((c = getopt_long(argc, argv, "h?VW",
 			  long_options, (int *) 0)) != EOF) {
 #else
-  while ((c = getopt(argc, argv, "h?v")) != EOF) {
+  while ((c = getopt(argc, argv, "h?VW")) != EOF) {
 #endif
     switch (c) {
       case 0  : break;
       case 'h':
       case '?': usage(); return 0;
-      case 'v': fprintf(stderr,"%s version 1.0\n", progname); return 1;
+      case 'V': fprintf(stderr,"%s version %s\n", progname, VERSION);
+		return 1;
+      case 'W': warranty(); return 1;
       default : break;
     }
   }
 
   setlocale(LC_CTYPE, ""); /* No "," and such BS for decimal points */
 
-  fprintf(stderr,"%s version %s. Copyright (c) 1993, 1997 David Frey.\n",
+  fprintf(stderr,"%s version %s. Copyright (c) 1993--1998 David Frey.\n",
 	  progname, VERSION);
   fprintf(stderr,"This is free software with ABSOLUTELY NO WARRANTY.\n");
 
@@ -413,14 +427,14 @@ int main(int argc, char *argv[])
 
     interactive=isatty(0); line=NULL;
 
-#ifdef HAVE_READLINE    
+#ifdef HAVE_LIBREADLINE
     initialize_readline(interactive);
 #endif
     if (interactive) {
       fprintf(stderr,"For details, type `warranty'.\n");
       fprintf(stderr,"Type `quit' to quit and `?' to get a summary.\n");
 
-#ifdef HAVE_READLINE    
+#ifdef HAVE_LIBHISTORY
       using_history();
 #endif
     }
@@ -429,20 +443,22 @@ int main(int argc, char *argv[])
       line=getline();
       quit=(line == NULL);
       if (!quit) {
+	char *s=line;
+
 	/* Skip white spaces */
-	while (*line != '\0' && isspace(*line)) line++;
-	if (*line != '\0') {
-#ifdef HAVE_READLINE          
-	  add_history(line);
+	while (*s != '\0' && isspace(*s)) s++;
+	if (*s != '\0') {
+#ifdef HAVE_LIBHISTORY
+	  add_history(s);
 #endif
-	  quit=eval(line);
+	  quit=eval(s);
 	  if (!quit) (void)showstack();
 	}
       }
       free(line); line=NULL; /* line was malloc'd by readline/getline. */
     }
     while (!quit);
-#ifdef HAVE_READLINE 
+#ifdef HAVE_LIBHISTORY
     remove_history();
 #endif    
   }  
